@@ -48,7 +48,7 @@ class CosTool:
 
     _path = None
     _disabled = False
-    query_type = None  # type: Union[QueryType, None]
+    query_type: Union[QueryType, None] = None
 
     def __init__(self, default_query_type: Optional[QueryType] = None):
         self.query_type = default_query_type
@@ -187,12 +187,9 @@ class CosTool:
         arch = "amd64" if arch == "x86_64" else arch
         res = "cos-tool-{}".format(arch)
         try:
-            path = Path(res).resolve()
-            path.chmod(0o777)
+            path = Path(res).resolve(strict=True)
             return path
-        except NotImplementedError:
-            logger.debug("System lacks support for chmod")
-        except FileNotFoundError:
+        except (FileNotFoundError, OSError):
             logger.debug('Could not locate cos-tool at: "{}"'.format(res))
         return None
 
