@@ -1,8 +1,10 @@
 # Copyright 2021 Canonical Ltd.
 # See LICENSE file for licensing details.
 """Types used by cos-lib."""
-from typing import Dict, List, Literal, Optional, TypedDict, Union
 
+from typing import Any, Dict, List, Literal, Optional, TypedDict, Union
+
+from ops.framework import StoredDict, StoredList
 from typing_extensions import Required
 
 QueryType = Literal["logql", "promql"]
@@ -43,3 +45,20 @@ class OfficialRuleFileFormat(TypedDict):
     """
 
     groups: List[OfficialRuleFileItem]
+
+
+def type_convert_stored(
+    obj: Union[StoredList, StoredDict, Any],
+) -> Union[List[Any], Dict[Any, Any], Any]:
+    """Helper for converting Stored[Dict|List|Set] to the objects they pretend to be.
+
+    Ref: https://github.com/canonical/operator/pull/572
+    """
+    if isinstance(obj, StoredList):
+        return list(map(type_convert_stored, obj))
+    if isinstance(obj, StoredDict):
+        rdict: Dict[Any, Any] = {}
+        for k in obj.keys():
+            rdict[k] = type_convert_stored(obj[k])
+        return rdict
+    return obj
